function settings=ravemakemodelrse(src,ev,settings)
%creates an RSE based on the settings matrix.
%must do three things:
%1) Fit the model.
%2) Assess the model against all rows and calculate predicted values. Store
%   these in settings.predictedvalues.
%3) Make a cell array of strings to be printed in a text file to make a
%   valid matlab function to calculate this model. (this is the hard part).

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

xcol=settings.inputs;
d=settings.dataset;
termmatrix=settings.termmatrix;

fitdata=datasets.alldata{d}(settings.fitrows,xcol);
fittodata=datasets.alldata{d}(settings.fitrows,settings.output);

switch settings.transformby
    case 'exp(y)'
        fittodata=exp(fittodata);
    case 'log(y)'
        fittodata=log(fittodata);
    case '1/y'
        fittodata=1./fittodata;
end
        

%multiply fit data by termmatrix:
nrows=size(fitdata,1);
nterms=size(termmatrix,1);
terms=zeros(nrows,nterms);
for c=1:nterms
    terms(:,c)=prod(bsxfun(@power,fitdata,termmatrix(c,:)),2);
end

%remove constant cols to avoid rank deficiency:
termtest=~all(bsxfun(@minus,terms,terms(1,:))==0,1);
if settings.includeintercept
    termtest(1)=true; %it gets set to false by the check above
end

fitterms=terms(:,termtest);

%do least squares fit:
B=fitterms\fittodata;

%replace constant terms:
b=zeros(numel(termtest),1);
b(termtest')=B;

%calculate predictions:
alldata=datasets.alldata{d}(:,xcol);
nrows=size(alldata,1);
nterms=size(termmatrix,1);
predterms=zeros(nrows,nterms);
for c=1:nterms
    predterms(:,c)=prod(bsxfun(@power,alldata,termmatrix(c,:)),2);
end

predictedvalues=(b'*predterms')';



switch settings.transformby
    case 'exp(y)'
        predictedvalues=real(log(predictedvalues));
    case 'log(y)'
        predictedvalues=exp(predictedvalues);
    case '1/y'
        predictedvalues=1./predictedvalues;
end
     
settings.predictedvalues=predictedvalues;


%make model string:
varnames=handles.allnames{d}(xcol);
varnames=ravemakevariablenames(varnames);

line1='xdata = [';
for v=1:numel(varnames);
    line1=[line1,varnames{v},' ']; %#ok<AGROW>
end

%Make matlab string:
str={};
str{end+1} = [line1(1:end-1),'];'];
str{end+1} = ['pwrs = ',mat2str(termmatrix),';'];
str{end+1} = ['coeffs = ',mat2str(b),';'];
str{end+1} = 'terms=zeros(size(xdata,1),size(pwrs,1));';
str{end+1} = 'for q=1:size(pwrs,1)';
str{end+1} = '    terms(:,q)=prod(bsxfun(@power,xdata,pwrs(q,:)),2);';
str{end+1} = 'end';
str{end+1} = 'y = (coeffs''*terms'')'';';

%Make plaintext string:
plainstr='';

for i = 1:size(termmatrix,1)
    plainstr = [plainstr sprintf('%16.16f*',b(i))];
    for j = 1:size(termmatrix,2)
        if termmatrix(i,j)>0
        plainstr = [plainstr sprintf('%s^%1.1u*',varnames{j},termmatrix(i,j))];        
        end
        
    end
    plainstr(end) = '+';
end

plainstr = plainstr(1:end-1);
plainstr = strrep(plainstr,'+-','-');



switch settings.transformby
    case 'exp(y)'
        str{end+1} = 'y = real(log(y));';
        %fittodata=exp(fittodata);
        plainstr = ['log(',plainstr,')'];
    case 'log(y)'
        %fittodata=log(fittodata);
        str{end+1} = 'y = exp(y);';
        plainstr = ['exp(',plainstr,')'];
    case '1/y'
        %fittodata=1/fittodata;
        str{end+1} = 'y = 1./y;';
        plainstr = ['1/(',plainstr,')'];

end
        

settings.modelstring=str';
settings.plaintxtstring=plainstr;

try %Symbolic toolbox required:
    symstr=sym(['y=',plainstr]);
    str=char(symstr);
    
    str=strrep(str,'==','='); %new versions of matlab do this.    
    eqidx=strfind(str,'=');
    str=strtrim(str(eqidx(1)+1:end));    
    settings.plaintxtstring=str;
    
    cstr=ccode(symstr);
    eqidx=strfind(cstr,'=');
    cstr=strtrim(cstr(eqidx(1)+1:end));    
    settings.cstring=cstr;
    
    ltx=latex(symstr);
    eqidx=strfind(ltx,'=');
    ltx=strtrim(ltx(eqidx(1)+1:end));    
    settings.latexstring=ltx;
end


