function settings=ravemakemodelffneuralnet(src,ev,settings)
%creates a neural net based on the settings matrix.
%must do three things:
%1) Fit the model.
%2) Assess the model against all rows and calculate predicted values. Store
%   these in settings.predictedvalues.
%3) Make a cell array of strings to be printed in a text file to make a
%   valid matlab function to calculate this model. (this is the hard part).

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

xcol=settings.inputs;
d=settings.dataset;
%fitdata=datasets.alldata{d}(settings.fitrows,xcol);
%fittodata=datasets.alldata{d}(settings.fitrows,settings.output);

xfitdata=datasets.alldata{d}(:,xcol);

if settings.fitindividually
    yfitdata=datasets.alldata{d}(:,settings.output);
else
    yfitdata=datasets.alldata{d}(:,settings.alloutputs);
end

%normalize data: feedforward nets like data normalized from -1 to 1.
xmin=min(xfitdata);
xrange=max(xfitdata)-min(xfitdata);

ymin=min(yfitdata);
yrange=max(yfitdata)-min(yfitdata);

xfitdata=bsxfun(@minus,xfitdata,xmin);
xfitdata=bsxfun(@rdivide,xfitdata,xrange);

yfitdata=bsxfun(@minus,yfitdata,ymin);
yfitdata=bsxfun(@rdivide,yfitdata,yrange);

settings.inputscalefactors=[xmin;xrange];
settings.responsescalefactors=[ymin;yrange];

%trainbr likes everything scaled to -1 to 1 instead of 0 to 1:
switch settings.trainalg
    case 'trainbr'
        xfitdata=xfitdata*2-1;
        yfitdata=yfitdata*2-1;
end

%xfitdata=xfitdata.*2-1;
%yfitdata=yfitdata.*2-1;

%check nn toolbox version:
v=ver('nnet');
if isempty(v)
    errordlg('This model type requires the MATLAB neural network toolbox, which does not appear to be installed. This toolbox can be purchased from The Mathworks.','Toolbox Required')
end



%% form model:

%does a new network need to be created?
if settings.fitindividually || settings.output == settings.alloutputs(1);
    
    idx=1; %index of this resposne into columns of networks predicted values
    
    switch v.Release
        case {'(R2007a)','(R2007b)','(R2008a)','(R2008b)','(R2009a)','(R2009b)','(R2010a)'}
            net=newff(xfitdata',yfitdata',settings.nhiddennodes,{settings.transferfcn},settings.trainalg,settings.learnalg);
        case {'(R2010b)','(R2011a)','(R2011b)','(R2012a)','(R2012b)'}
            net=feedforwardnet(settings.nhiddennodes);
        otherwise
            net=feedforwardnet(settings.nhiddennodes);
            
    end
    
    %set training parameters:
    net.trainFcn=settings.trainalg;
    f=fieldnames(settings.trainParam);
    for s=1:numel(f)
        net.trainParam.(f{s})=settings.trainParam.(f{s});
    end
    
    if isfield(settings,'efficiency')
        f=fieldnames(settings.efficiency);
        for s=1:numel(f)
            net.efficiency.(f{s})=settings.efficiency.(f{s});
        end
        
    end
    
    %set up validation data:
    net.divideFcn='divideind';
    net.divideParam.trainInd=find(settings.fitrows);
    net.divideParam.valInd=find(settings.validaterows);
    net.divideParam.testInd=find(settings.testrows);
    
    %other parameters:
    net.layers{1}.transferFcn=settings.transferfcn;
    net.trainParam.showWindow=0;
    net.trainParam.showCommandLine=1;
    net.performFcn='sse'; %sum squared error
    
    %remove processing steps:
    net.inputs{1}.processFcns={};
    net.outputs{2}.processFcns={};
    
    if settings.doregularization && ~strcmp(settings.trainalg,'trainbr') %bayesian regularization handles its own regularization
        net.performFcn='msereg';
        net.performParam.ratio=settings.regularizationvalue;
    end
    
    %configure network if necessary:
    switch v.Release
        case {'(R2010b)','(R2011a)','(R2011b)','(R2012a)','(R2012b)'}
            net=configure(net,xfitdata',yfitdata');
    end

    %do training:
    net=init(net);
    net=train(net,xfitdata',yfitdata');
    
else
    %get network from the first model in this group:
    idx=find(settings.output==settings.alloutputs);
    nmod=numel(handles.models);
    net=handles.models{nmod-idx+2}.netobj;
end

settings.netobj=net;

%calculate predictions:
alldata=datasets.alldata{d}(:,xcol);

%normalize:
alldata=bsxfun(@minus,alldata,xmin);
alldata=bsxfun(@rdivide,alldata,xrange);

switch settings.trainalg
    case 'trainbr'
        alldata=alldata*2-1;
end

preds=sim(net,alldata')';

%denormalize:
switch settings.trainalg
    case 'trainbr'
        preds=(preds/2+0.5);
end

preds=bsxfun(@times,preds,yrange);
preds=bsxfun(@plus,preds,ymin);

settings.predictedvalues=preds(:,idx);


%% make model string:
varnames=handles.allnames{d}(xcol);
varnames=ravemakevariablenames(varnames);

line1='xdata = [';
for v=1:numel(varnames);
    varnames{v}=strrep(varnames{v},' ',''); %remove spaces
    line1=[line1,varnames{v},' ']; %#ok<AGROW>
end

str{1}=[strtrim(line1),'];'];

%normalization values:
str{2}=['xmin=',mat2str(xmin),';'];
str{3}=['xrange=',mat2str(xrange),';'];

str{4}=['ymin=',mat2str(ymin(idx)),';'];
str{5}=['yrange=',mat2str(yrange(idx)),';'];

str{end+1}='xdata=bsxfun(@minus,xdata,xmin);';
str{end+1}='xdata=bsxfun(@rdivide,xdata,xrange);';

%do additional normalization:
switch settings.trainalg
    case 'trainbr'
        str{end+1}=['xdata=(xdata*2-1);'];
end

%make logsig:

str{end+1}='logsig=@(n) 1./(1 + exp(-n));';

%form neural net values:
str{end+1}=['IW=',mat2str(net.IW{1}),';'];
str{end+1}=['LW=',mat2str(net.LW{2,1}(idx,:)),';'];
str{end+1}=['b=',mat2str(net.b{1}),';'];
str{end+1}=['b0=',mat2str(net.b{2}(idx)),';'];

%do calculation:
calcstr='y=b0';

for k=1:settings.nhiddennodes %for each hidden node... 
   calcstr=[calcstr,' + LW(',num2str(k),')*logsig(b(',num2str(k),')+IW(',num2str(k),',:)*xdata'')'] ; %#ok<AGROW>
end

str{end+1}=[calcstr,';'];

%denormalize:
switch settings.trainalg
    case 'trainbr'
        str{end+1}='y=(y/2+0.5);';
end

str{end+1}='y=y''*yrange+ymin;';

settings.modelstring=str';

normvarnames=varnames;
for n=1:numel(normvarnames)
   normvarnames{n}=['((',varnames{n},'-',num2str(xmin(n),10),')/',num2str(xrange(n),10),')',]; 
end

%make plaintext string:
str=[num2str(net.b{2}(idx),10),'+'];
for k=1:settings.nhiddennodes
   innerstr=[num2str(net.b{1}(k),10),'+'];
   for g=1:numel(varnames)
       innerstr=[innerstr,num2str(net.IW{1}(k,g),10),'*',normvarnames{g},'+'];       
   end
   innerstr=innerstr(1:end-1); %remove trailing +
   
   str=[str,num2str(net.LW{2,1}(idx,k),10),'*(1/(1+exp(-1*(',innerstr,'))))+']; 
end
str=str(1:end-1); %remove trailing +

str=[num2str(ymin(idx),10),'+',num2str(yrange(idx),10),'*(',str,')'];
str=strrep(str,'+-','-');

settings.plaintxtstring=str;

try %Symbolic toolbox required:
    symstr=sym(['y=',str]);
    str=char(symstr);
    
    str=strrep(str,'==','='); %new versions of matlab do this.    
    eqidx=strfind(str,'=');
    str=strtrim(str(eqidx(1)+1:end));    
    settings.plaintxtstring=str;
    
    cstr=ccode(symstr);
    eqidx=strfind(cstr,'=');
    cstr=strtrim(cstr(eqidx(1)+1:end));    
    settings.cstring=cstr;
    
    ltx=latex(symstr);
    eqidx=strfind(ltx,'=');
    ltx=strtrim(ltx(eqidx(1)+1:end));    
    settings.latexstring=ltx;
end






