function varargout = printVariable(ins,varname)
%Prints a script (as a cell array) that recreates the input variable.
%Similar functionality as mat2str, but works for general variables.

persistent ishg2
if isempty(ishg2)
    %New graphics engine or old one?
    try % a "try" because i'm not sure when verLessThan was introduced.
        if verLessThan('matlab','8.4')
            ishg2 = false;
        else
            ishg2 = true;
        end
    catch
        ishg2 = false;
    end
end




if nargin<2
    varname=inputname(1);
end

outs=processVariable(ins,varname,{});

%Print to screen instead?
if nargout==0
    fprintf('\n');
    for i = 1:numel(outs)
        fprintf('    %s\n',outs{i});
    end
else
    varargout{1}=outs;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function outs=processVariable(ins,varname,prior)


if isnumeric(ins) || islogical(ins) || ischar(ins)
    outs=processNumeric(ins,varname);
elseif iscell(ins)
    outs=processCell(ins,varname);
elseif isstruct(ins)
    
    %check if it's a vector structure:
    
    if numel(ins)==1 %scalar struct:
        outs=processStruct(ins,varname);
    else
        outs={};
        dims=size(ins);
        for i = 1:prod(dims)
            pos=ind2sub(dims,i);
            %construct cell index string:
            sfx='(';
            for j=1:numel(pos)
                sfx=[sfx,sprintf('%0.0u',pos(j)),',']; %#ok<AGROW>
            end
            sfx(end)=')';
            outs=processVariable(ins(i),[varname,sfx],outs);
        end
    end
    
elseif isa(ins,'function_handle');
    outs=[varname,' = str2func(',func2str(ins),');'];
else
%     warning(['Skipping field ',varname,' because it contains an unprocessable data class']);
    outs={};
end

outs=[prior;outs];

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [outs]=processNumeric(ins,varname)
if numel(size(ins))<3 %if this is a "flat" variable
if islogical(ins)
    outs={[varname,' = logical(',mat2str(double(ins)),');']};
else
    outs={[varname,' = ',mat2str(ins,16),';']};
end
else %Write each page separately:
    dims=size(ins);
    outs={};
     for i = prod(dims(3:end)):-1:1
        posses=cell(1,numel(dims(3:end)));
        [posses{:}]=ind2sub(dims(3:end),i);
        %construct cell index string:
        sfx='(:,:,';
        for j=1:numel(posses)
            sfx=[sfx,sprintf('%0.0u',posses{j}),',']; %#ok<AGROW>
        end
        sfx(end)=')';
        outs=processVariable(ins(:,:,i),[varname,sfx],outs);
    end
    
    
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [outs]=processStruct(ins,varname)
outs={};

if isempty(ins)
    outs=[varname,' = struct();'];
else
    
    fields = fieldnames(ins);
    for i = 1:numel(fields)
        thisfield=fields{i};
        outs=processVariable(ins.(thisfield),[varname,'.',thisfield],outs);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [outs]=processCell(ins,varname)
outs={};
dims=size(ins);

%if nothing in this cell is itself a cell, use simplified syntax:
basictest = cellfun(@isbasictype,ins);

if isempty(ins)
    outs=[varname,' = {};'];
elseif all(basictest(:)) && numel(dims)<3 %plain 2-D cell array contains no nested cells, print it in a single line:
    allcontents = cellfun(@mat2str,ins,'uniformoutput',false);
    %put a comma after each cell:
    allcontents(:,1:end-1) = cellfun(@(x) [x,','],allcontents(:,1:end-1),'uniformoutput',false);
    %change end of rows to a semicolon:
    allcontents(1:end-1,end) = cellfun(@(x) [x,';'],allcontents(1:end-1,end),'uniformoutput',false);
    allcontents=allcontents';
    outs=[varname,' = {',allcontents{:},'};'];
else %cell array contains nested cells, or is high dimensional. Print each cell on its own line:
    for i = 1:prod(dims)
        pos=ind2sub(dims,i);
        %construct cell index string:
        sfx='{';
        for j=1:numel(pos)
            sfx=[sfx,sprintf('%0.0u',pos(j)),',']; %#ok<AGROW>
        end
        sfx(end)='}';
        outs=processVariable(ins{i},[varname,sfx],outs);
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function result=isbasictype(x)
%test whether input is a basic data type that can be processed with
%mat2str:
result = isnumeric(x) || ischar(x) || islogical(x);






