function varargout=rave(varargin)
% Welcome to the Rave.
% For help with Rave you should read the
% documentation at http://www.rave.gatech.edu/help/
%
% Quick help: type "rave" at the MATLAB command prompt to start the program.
% If you haven't used Rave before, an installer will run.
% Then click Manage Data Sets to load a data set (must be "flat file" format).
% Then click "Create New..." to make a graph, or go the Model tab to load a
% model or make a surrogate model. Once you have loaded a model, you can
% run optimizers from the Optimize tab.
%
% Rave Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% If you would like to obtain Rave under a different license, contact
% mdaskilewicz@asdl.gatech.edu
%
% Rave was created by Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu
% Please contact me with bug reports, feature requests, collaboration
% ideas, or feedback on how you are using RAVE.

%New graphics engine or old one?
try % a "try" because i'm not sure when verLessThan was introduced.
    if verLessThan('matlab','8.4')
        handles.ishg2 = false;
    else
        handles.ishg2 = true;
    end
catch
    handles.ishg2 = false;
end

raveroot=which('rave');
raveroot=raveroot(1:end-6);
addpath(genpath(raveroot))

warning('off','MATLAB:HandleGraphics:ObsoletedProperty:JavaFrame');
warning('off','MATLAB:hg:willberemoved');

stops=dbstatus;
clear functions %attempt to avoid errors due to old stuff already being in memory
dbstop(stops);

if ~usejava('swing') || ~usejava('awt')
    errordlg('Rave requires Java to run.');
    return
end

set(0,'showhiddenhandles','off')

if isempty(which('raveprefs.mat'));
    raveinstall;
    return;
end

load raveprefs

raveprefs=ravevalidateprefs(raveprefs); %#ok<NODEF>
% raveprefs.wswidth=1350;
% raveprefs.wsheight=1000;

raveprefs.wswidth= 5376;
raveprefs.wsheight= 3360;

thisver=ravedatenum;
if thisver>raveprefs.currentversion;raveprefs.currentversion=thisver;end
save(which('raveprefs.mat'),'raveprefs');

if raveprefs.isportable
    raveprefs.defaultdir=fullfile(raveroot,'ravefiles');
end

if isempty(which('RaveTableSorter'))
    javaaddpath(which('RaveJavaClasses.jar'));
    javaaddpath(which('jlatexmath-1.0.2.jar')); %equation display in html panes
    javaaddpath(which('PFLP.jar')); %point labelling 
end

%reset default font sizes to ensure things fit on the screen:
set(0,'defaulttextfontsize',10)
set(0,'defaultuicontrolfontsize',8);
if ispc
    set(0,'defaultuicontrolfontname','ms sans serif')
elseif ismac
    set(0,'defaultuicontrolfontname','geneva')
end
set(0,'defaultuicontrolhittest','off'); %maybe speeds up the hittest function?

%The default font has always been Helvetica, but in 2014b matlab uses a different font
%to draw helvetica, so restore it to the old fall-back font:
set(0,'defaulttextfontname','ms sans serif')


%check if monitor position has changed:
mpos=get(0,'MonitorPositions');
if numel(mpos(:))~=numel(raveprefs.monitorpositions(:)) || ~all(mpos(:)==raveprefs.monitorpositions(:))
    answer=questdlg('Your screen resolution has changed since you last used Rave. Do you want to resize the Rave window to better fit your new resolution?','Resize Rave Window');
    drawnow;
    if strcmpi(answer,'yes')
        hhh=raveresizewindow([],[]);
        uiwait(hhh);
        load raveprefs
        raveprefs.monitorpositions=get(0,'MonitorPositions');
    end
end
drawnow;
raveprefs.monitorpositions=get(0,'MonitorPositions');
save(which('raveprefs.mat'),'raveprefs');

handles.prefs=raveprefs;
handles.colors=raveprefs.colors;
handles.fonts=raveprefs.fonts;
handles.defaultdir=raveprefs.defaultdir;

%make tray icon:
sysTray = java.awt.SystemTray.getSystemTray;
if (sysTray.isSupported)
    raveroot=which('rave');
    raveroot=raveroot(1:end-6);
    myIcon = fullfile(raveroot,'images','tinyrave.png');
    iconImage = java.awt.Toolkit.getDefaultToolkit.createImage(myIcon);
    handles.trayIcon = java.awt.TrayIcon(iconImage, 'initial tooltip');
    handles.trayIcon.setToolTip('Rave is running.');
    sysTray.add(handles.trayIcon)
else
    handles.trayIcon=[];
end

%Set starting renderer based on matlab graphics version:
if handles.ishg2
    render='opengl';
else
    render='painters';
end

%Make figure:
handles.fig=figure('visible','on','toolbar','none','name','Rave','menubar','none','numbertitle','off',...
    'resize','off','color',handles.colors.frame,'units','pixels','renderermode','manual','renderer',render,...
    'handlevisibility','callback','integerhandle','off','doublebuffer','on');

if handles.ishg2
    set(handles.fig,'defaultaxessortmethod','childorder')
end


%opengl renderer settings:
if ispc && ~handles.ishg2
    if ~isempty(handles.prefs.wvisual);set(handles.fig,'wvisual',handles.prefs.wvisual);end
elseif isunix
    if ~isempty(handles.prefs.xvisual);set(handles.fig,'xvisual',handles.prefs.xvisual);end
end

if nargout==1
    varargout={handles.fig};
else
    varargout=cell(0);
end

% outerwidth=handles.prefs.preferredposition(3); %new
% outerheight=handles.prefs.preferredposition(4); %new


%set work dir to default dir to start.
if ~strcmp(handles.defaultdir(end),filesep) %if it doesnt end in a \, put one there.
    handles.defaultdir(end+1)=filesep;
end

%default colormap:
guidata(handles.fig,handles);
[cmap,cmapnames]=ravemakecolormap(handles.fig);

set(handles.fig,'colormap',cmap);
handles.cmapnames=cmapnames;

set(handles.fig,'position',handles.prefs.preferredposition)

%set up data structure for passing info between windows:
setappdata(handles.fig,'backupdata',{});

%
%
%
%Initialize handles fields:
handles=raveinitializehandles(handles);
guidata(handles.fig,handles);
%
%
%
%


%% Load all plugins %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Load data for workspace objects:
raveupdategraphtypes(handles.fig,[]);

%Load data for optimizers:
raveupdateoptimizertypes(handles.fig,[])

%Load data for surrogate models:
raveupdatemodeltypes(handles.fig,[])

%Load data for explore methods:
raveupdateexploretypes(handles.fig,[]);

%Load data for distributions:
raveupdatedistributiontypes(handles.fig,[])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%
handles=guidata(handles.fig);
try %#ok<TRYNC> %see if system will let you save the path.
    savepath
end

%%
innerposition=get(handles.fig,'position');
handles.width=innerposition(3);
handles.height=innerposition(4);

%% make main data table
guidata(handles.fig,handles);
ravecreatemaintable(handles.fig,[]);
handles=guidata(handles.fig);

set(handles.maintable.handle,'visible','off');

%% set up main (workspace) axes

% handles.viewpane=uipanel('position',[0 0 .3 .3],'backgroundcolor',[1 0 1],'units','pixels',...
%     'foregroundcolor',handles.colors.frame,'parent',handles.fig,'highlightcolor',handles.colors.frame,'shadowcolor',handles.colors.frame,'hittest','off');

handles.boneyard=uipanel('position',[0 0 .3 .3],'backgroundcolor',[1 1 1],'units','pixels',...
    'foregroundcolor',handles.colors.frame,'parent',handles.fig,'highlightcolor',handles.colors.frame,'shadowcolor',handles.colors.frame,...
    'visible','off','hittest','off','tag','boneyard');
handles.wspanel=uipanel('position',[0 0 .3 .3],'backgroundcolor',[0 1 1],'units','pixels',...
    'foregroundcolor',handles.colors.frame,'parent',handles.fig,'highlightcolor',handles.colors.frame,'shadowcolor',handles.colors.frame,...
    'hittest','off','bordertype','none','tag','wspanel');

handles.wscols=handles.prefs.wswidth/handles.prefs.gridsize;
handles.wsrows=handles.prefs.wsheight/handles.prefs.gridsize;


if handles.ishg2
    set(handles.wspanel,'defaultaxessortmethod','childorder')
end


%% set up frame axes
handles.leftmargin=axes('units','pixels','position',[0 -5 30+315*handles.prefs.tabscale handles.height+11],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame);

handles.rightmargin=axes('units','pixels','position',[handles.width-14 -5 40 handles.height+20],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame);

handles.topmargin=axes('units','pixels','position',[-5 handles.height-35 handles.width+5 60],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame,'xlimmode','manual','ylimmode','manual',...
    'xlim',[0 1],'ylim',[0 1]);
handles.bottommargin=axes('units','pixels','position',[-5 0 handles.width+10 16],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xcolor',handles.colors.frame,'ycolor',handles.colors.frame,'zcolor',handles.colors.frame,'color',handles.colors.frame);

leftpos=get(handles.leftmargin,'position');
rightpos=get(handles.rightmargin,'position');
toppos=get(handles.topmargin,'position');
botpos=get(handles.bottommargin,'position');

% handles.workspacedims=[rightpos(1)-leftpos(1)-leftpos(3),toppos(2)-botpos(2)-botpos(4)];
% handles.workspacepos=[1 1 handles.workspacedims(1) handles.workspacedims(2)];

wsorigin = [leftpos(1)+leftpos(3) botpos(2)+botpos(4)];
wspos = [wsorigin(1) wsorigin(2) rightpos(1)-1-wsorigin(1) toppos(2)-1-wsorigin(2)];




% wspos = [350 16 1312 965];
% set(handles.wspanel,'position',[1 putpanelhere(2)-3 putpanelhere(3)+5 putpanelhere(4)+5],'userdata',[1 putpanelhere(2)-3 putpanelhere(3)+5 putpanelhere(4)+5]);

set(handles.wspanel,'position',wspos,'userdata',[350 16 1312 965]);
% set(handles.mainaxes,'position',[0 0 wspos(3)+1 wspos(4)+1]); %shift down/left one pixel so that the axes lines are always hidden


set(handles.boneyard,'position',get(handles.wspanel,'position'));
% set([handles.mainaxes handles.foregroundaxes],'parent',handles.wspanel);
% set([handles.mainaxes handles.foregroundaxes],'position',[1 2 putpanelhere(3) putpanelhere(4)])


%make infobar:
infopos=get(handles.topmargin,'position');
infoaxes=axes('units','pixels','position',infopos+[360 0 -360 0],'color','none','visible','off','parent',handles.fig,'tag','infoaxes','hittest','off');
handles.infobar=text(.0,.05,'','horizontalalignment','left',...
    'fontsize',handles.fonts.tabsize*0.8,'fontname',handles.fonts.tab,'color',[.3 .6 .8],'parent',infoaxes,'verticalalignment','baseline','interpreter','tex','hittest','off');

handles.mainaxes=axes('units','pixels','position',[1 1 wspos(3) wspos(4)],...
    'parent',handles.wspanel,'ytickmode','manual','yticklabel','','xtickmode',...
    'manual','xticklabel','','box','off','clipping','off',...
    'xlimmode','manual','ylimmode','manual','color',handles.colors.wsbg,'buttondownfcn',@ravedragview,'tag','mainaxes',...
    'handlevisibility','on','xcolor',handles.colors.wsbg,'ycolor',handles.colors.wsbg','zcolor',handles.colors.wsbg);

handles.foregroundaxes=axes('units','pixels','position',[0 0 wspos(3) wspos(4)],...
    'parent',handles.wspanel,'ytickmode','manual','yticklabel','','xtickmode',...
    'manual','xticklabel','','box','off','clipping','off',...
    'xlimmode','manual','ylimmode','manual','color','none','hittest','off','visible','off','tag','foregroundaxes');

%  axessize=get(handles.mainaxes,'position');
% axessize(3:4)=handles.prefs.gridsize.*floor(axessize(3:4)./handles.prefs.gridsize);
% set(handles.mainaxes,'position',axessize);

%draw grids
% set(handles.mainaxes,'xlim',[0 axessize(3)],'ylim',[0 axessize(4)])

% handles.prefs.wswidth=axessize(3);
% handles.prefs.wsheight=axessize(4);

r=0:(handles.prefs.gridsize/4):handles.prefs.wsheight;
xcoords=repmat([0 handles.prefs.wswidth handles.prefs.wswidth 0],1,ceil(numel(r)./2));
ycoords=[r;r];
ycoords=ycoords(:);
xcoords=xcoords(1:numel(ycoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','sgrid');

r=0:(handles.prefs.gridsize/2):handles.prefs.wsheight;
xcoords=repmat([0 handles.prefs.wswidth handles.prefs.wswidth 0],1,ceil(numel(r)./2));
ycoords=[r;r];
ycoords=ycoords(:);
xcoords=xcoords(1:numel(ycoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','mgrid');

r=0:handles.prefs.gridsize:handles.prefs.wsheight;
xcoords=repmat([0 handles.prefs.wswidth handles.prefs.wswidth 0],1,ceil(numel(r)./2));
ycoords=[r;r];
ycoords=ycoords(:);
xcoords=xcoords(1:numel(ycoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','lgrid');

c=0:(handles.prefs.gridsize/4):handles.prefs.wswidth;
ycoords=repmat([0 handles.prefs.wsheight handles.prefs.wsheight 0],1,ceil(numel(c)./2));
xcoords=[c;c];
xcoords=xcoords(:);
ycoords=ycoords(1:numel(xcoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','sgrid');

c=0:(handles.prefs.gridsize/2):handles.prefs.wswidth;
ycoords=repmat([0 handles.prefs.wsheight handles.prefs.wsheight 0],1,ceil(numel(c)./2));
xcoords=[c;c];
xcoords=xcoords(:);
ycoords=ycoords(1:numel(xcoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',(handles.colors.wsgrid+handles.colors.wsbg)/2,'parent',handles.mainaxes,'hittest','off','tag','mgrid');

c=0:(handles.prefs.gridsize):handles.prefs.wswidth;
ycoords=repmat([0 handles.prefs.wsheight handles.prefs.wsheight 0],1,ceil(numel(c)./2));
xcoords=[c;c];
xcoords=xcoords(:);
ycoords=ycoords(1:numel(xcoords));
line(xcoords,ycoords,-1*ones(size(ycoords)),'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','lgrid');

handles.lgrid=findobj(handles.mainaxes,'tag','lgrid');
handles.mgrid=findobj(handles.mainaxes,'tag','mgrid');
handles.sgrid=findobj(handles.mainaxes,'tag','sgrid');

if handles.ishg2
    set(handles.lgrid,'alignvertexcenters','on');
    set(handles.mgrid,'alignvertexcenters','on');
    set(handles.sgrid,'alignvertexcenters','on');
end


%fix grid visibility
if handles.prefs.gridmode==1
    set(handles.lgrid,'visible','on')
    set(handles.mgrid,'visible','off')
    set(handles.sgrid,'visible','off')
    handles.snapsize=handles.prefs.gridsize;
elseif handles.prefs.gridmode==2
    set(handles.lgrid,'visible','on')
    set(handles.mgrid,'visible','on')
    set(handles.sgrid,'visible','off')
    handles.snapsize=handles.prefs.gridsize/2;
else
    set(handles.lgrid,'visible','on')
    set(handles.mgrid,'visible','on')
    set(handles.sgrid,'visible','on')
    handles.snapsize=handles.prefs.gridsize/4;
end

if ~handles.prefs.snaptogrid
    handles.snapsize=1;
end

% handles.wsrows=axessize(4)/handles.prefs.gridsize;
% handles.wscols =axessize(3)/handles.prefs.gridsize;
% handles.wsdims=[axessize(3) axessize(4)];

%draw additional box around axes that is always visible
for r= [0 handles.prefs.wsheight]
    line([0 handles.prefs.wswidth],[r r],'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','wspanelborder');
end
for c= [0 handles.prefs.wswidth]
    line([c c],[0 handles.prefs.wsheight],'color',handles.colors.wsgrid,'parent',handles.mainaxes,'hittest','off','tag','wspanelborder');
end


%% make page controls:
handles.pageedit=uicontrol('style','edit','position',[280+5 handles.height-31 40 20],'parent',handles.fig,...
    'fontname','Arial','fontsize',16,'backgroundcolor',handles.colors.frame,'foregroundcolor',handles.colors.title,...
    'string','1','callback',@raveeditpage,'tag','pageedit');
ppp=findjobj(handles.pageedit);
ppp.setBorder(javax.swing.BorderFactory.createEmptyBorder)

lbutton=imread('leftbuttonup.png','backgroundcolor',handles.colors.frame);
rbutton=imread('rightbuttonup.png','backgroundcolor',handles.colors.frame);

handles.pagecontrols.left=uicontrol('units','pixels','style','pushbutton','position',[263+5 handles.height-35 17 27],'parent',handles.fig,...
    'cdata',lbutton,'buttondownfcn',{@ravechangepage,-1},'tag','pageleftbutton','enable','inactive','hittest','on','tooltip','Go to previous page of workspace');
handles.pagecontrols.right=uicontrol('units','pixels','style','pushbutton','position',[320+5 handles.height-35 17 27],'parent',handles.fig,...
    'cdata',rbutton,'buttondownfcn',{@ravechangepage,1},'tag','pagerightbutton','enable','inactive','hittest','on','tooltip','Go to next page of workspace');


%% set up workspace explorer axes
if handles.height>=900
    pushdown=30; %move down a little so its not so cramped
    explheight=150;
else
    pushdown=0;
    explheight=110;
end
if handles.wscols>=240/explheight*handles.wsrows %width constrained
    
    handles.explorer=axes('units','pixels','position',[100 handles.height-155-pushdown 240 handles.wsrows/handles.wscols*240],...
        'parent',handles.fig,...
        'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off','clipping','off',...
        'xlimmode','manual','ylimmode','manual','xlim',[0 handles.prefs.wswidth],...
        'ylim',[0 handles.prefs.wsheight],'ycolor',[1 1 1],'xcolor',[1 1 1],...
        'buttondownfcn',@ravemoveview,'hittest','on','color',handles.colors.wsbg);
    
else%height constrained
    handles.explorer=axes('units','pixels','position',[100 handles.height-5-explheight-pushdown handles.wscols/handles.wsrows*explheight explheight],...
        'parent',handles.fig,...
        'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off','clipping','off',...
        'xlimmode','manual','ylimmode','manual','xlim',[0 handles.prefs.wswidth],'ylim',[0 handles.prefs.wsheight],...
        'ycolor',[1 1 1],'xcolor',[1 1 1],...
        'buttondownfcn',@ravemoveview,'hittest','on','color',handles.colors.wsbg);
end


if ~handles.ishg2
    set(handles.explorer,'drawmode','fast')
else
    set(handles.explorer,'sortmethod','childorder')
end


%draw explorer grid
handles.explorergrid=copyobj(handles.lgrid,handles.explorer);
set(handles.explorergrid,'color',(handles.colors.wsgrid+handles.colors.wsbg)/2)
if handles.ishg2
    set(handles.explorergrid,'alignvertexcenter','on')
end

if handles.height<=776
    set(handles.explorer,'position',[-500 -500 100 100]);
end


%% draw current-view frame

%calculate where to put it.
wspos=get(handles.wspanel,'position');
wsheight = handles.prefs.wsheight;

% makethistall=min(handles.workspacepos(4),handles.prefs.wsheight);
% makethiswide=min(handles.workspacepos(3),handles.prefs.wswidth);
% putthisfarup=handles.workspacepos(2)-mainaxespos(2);
% putthisfarleft=handles.workspacepos(1)-mainaxespos(1);

%draw it
handles.currentview=rectangle('position',[1 wsheight-wspos(4) wspos(3) wspos(4)],'parent',handles.explorer,...
    'linewidth',1.5,'clipping','off','edgecolor',handles.colors.text,'hittest','off','tag','currentview');
if handles.ishg2
    set(handles.currentview,'alignvertexcenters','on')
end

%get x y coords of rectangle in explorer
cvpos=get(handles.currentview,'position');
set(handles.wspanel,'userdata',cvpos);


%% make explorer title if on large screen
aaa=axes('units','pixels','position',[355-handles.wscols/handles.wsrows*150 handles.height-1-pushdown-4 200 50],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off','clipping','off',...
    'xlimmode','manual','ylimmode','manual','xlim',get(handles.mainaxes,'xlim'),'ylim',get(handles.mainaxes,'ylim'),'ycolor',handles.colors.frame,'xcolor',handles.colors.frame,...
    'buttondownfcn','','hittest','off','visible','off','color','none');
handles.explorertitle=text(0,0,'Navigator','fontname',handles.fonts.title,'fontsize',1.125*handles.fonts.titlesize,'color',handles.colors.title,...
    'parent',aaa,'verticalalignment','baseline');

if handles.height<900
    set(handles.explorertitle,'visible','off')
end


%% make scrollbars
originalLnF = javax.swing.UIManager.getLookAndFeel;  %class
if ~handles.prefs.dontchangemylaf
    try %#ok<TRYNC>
        newLnF = 'com.sun.java.swing.plaf.nimbus.NimbusLookAndFeel';
        javax.swing.UIManager.setLookAndFeel(newLnF);
    end
end

vslidestep=[.05 cvpos(4)/((handles.prefs.gridsize*handles.wsrows)-cvpos(4))];

if any(vslidestep<0)|| any(isinf(vslidestep))
    handles.vscroll=uicontrol('position',[handles.width-16 17 16 handles.height-53],'style','slider',...
        'callback',{@vertscroll,handles.fig},'min',cvpos(4)/2,'max',cvpos(4)/2+eps(cvpos(4)),...
        'sliderstep',[.02 0.1],...
        'value',cvpos(4)/2,'parent',handles.fig,'enable','off');
else
    handles.vscroll=uicontrol('position',[handles.width-16 17 16 handles.height-53],'style','slider',...
        'callback',{@vertscroll,handles.fig},'min',cvpos(4)/2,'max',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2+5,...
        'sliderstep',vslidestep,...
        'value',handles.prefs.gridsize*handles.wsrows-cvpos(4)/2,'parent',handles.fig);
end
%horizontalslider
hslidestep=[.02 cvpos(3)/((handles.prefs.gridsize*handles.wscols)-cvpos(3))];
if any(hslidestep<0) || any(isinf(hslidestep))
    handles.hscroll=uicontrol('position',[345+1 1 handles.width-345-17 16],'style','slider',...
        'callback',{@horizscroll,handles.fig},'min',cvpos(3)/2,'max',cvpos(3)/2+eps(cvpos(3)),...
        'sliderstep',[.02 0.1],...
        'value',cvpos(3)/2,'parent',handles.fig,'enable','off');
else
    handles.hscroll=uicontrol('position',[345+1 1 handles.width-345-17 16],'style','slider',...
        'callback',{@horizscroll,handles.fig},'min',cvpos(3)/2,'max',handles.prefs.gridsize*handles.wscols-cvpos(3)/2+5,...
        'sliderstep',[.02 cvpos(3)/((handles.prefs.gridsize*handles.wscols)-cvpos(3))],...
        'value',cvpos(3)/2,'parent',handles.fig);
end
drawnow;
javax.swing.UIManager.setLookAndFeel(originalLnF);

set([handles.vscroll handles.hscroll],'handlevisibility','off');

%% set up title axes if on a large monitor
aaaa=axes('units','pixels','position',[20 20 320 40],'parent',handles.fig,...
    'visible','off','xlim',[0 1],'ylim',[0 1]);
handles.titleaxes=text(.5,0,'rave','fontname','magneto','parent',aaaa,'fontsize',80,'color',handles.colors.title,...
    'verticalalignment','baseline','backgroundcolor','none','hor','center','hittest','on','buttondownfcn',@ravefixtabs,...
    'clipping','off');
if handles.height<900
    set(handles.titleaxes,'visible','off')
    axesoffset=25;
else
    axesoffset=85;
end

%% set up tab axes

tabwidth=1; %pixel width of tab borders

if handles.height>=900
    pushup=15;
else
    pushup=0;
end

handles.backtabposition=[20 pushup-20+axesoffset-650*(handles.prefs.tabscale-1) 30*handles.prefs.tabscale 630*handles.prefs.tabscale];
handles.fronttabposition=[20+30*(handles.prefs.tabscale) 20+pushup-20+axesoffset-650*(handles.prefs.tabscale-1) 30*handles.prefs.tabscale 631*handles.prefs.tabscale];

handles.bgtabaxes=axes('units','pixels','position',[20+30*(handles.prefs.tabscale) pushup+axesoffset-20-650*(handles.prefs.tabscale-1) 261*handles.prefs.tabscale 631*handles.prefs.tabscale],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'clipping','off',...
    'ycolor',handles.colors.frame,'xcolor',handles.colors.frame,'zcolor',handles.colors.tabfg,'color','none','visible','on','hittest','off');
handles.tabaxes=axes('units','pixels','position',[20+60*(handles.prefs.tabscale) 20+pushup-20+axesoffset-650*(handles.prefs.tabscale-1) 261*handles.prefs.tabscale 631*handles.prefs.tabscale],'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'clipping','off',...
    'ycolor',handles.colors.frame,'xcolor',handles.colors.frame,'zcolor',handles.colors.tabfg,'color','none','visible','on','hittest','off');
handles.fronttabaxes=axes('units','pixels','position',handles.fronttabposition,'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'color','none',...
    'ycolor',[0 0 0],'xcolor',[0 0 0],'zcolor',handles.colors.tabbg,...
    'visible','off','hittest','off');
handles.backtabaxes=axes('units','pixels','position',handles.backtabposition,'parent',handles.fig,...
    'ytickmode','manual','yticklabel','','xtickmode','manual','xticklabel','','box','off',...
    'xlimmode','manual','ylimmode','manual','xlim',[0 1],'ylim',[0 1],'color','none',...
    'ycolor',[0 0 0],'xcolor',[0 0 0],'zcolor',handles.colors.frame,...
    'visible','off','hittest','off');

%set up non-tab tab borders
%background
patch([0 1 1 0],[1 1 0 0],handles.colors.tabbg,'parent',handles.tabaxes,'edgecolor','none','tag','tabbg','hittest','off')
patch([0 1 1 0],[1 1 0 0],handles.colors.tabbg,'parent',handles.bgtabaxes,'edgecolor','none','tag','tabbg','hittest','off')

tablines(1)=line([0 1],[0 0],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.tabaxes,'hittest','off');%bottom
tablines(2)=line([0 1],[1 1],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.tabaxes,'hittest','off');%top
tablines(3)=line([1 1],[0 1],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.tabaxes,'hittest','off');%right
tablines(4)=line([0 1],[0 0],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.bgtabaxes,'hittest','off');%subbottom
tablines(5)=line([1 1],[0 1],'linewidth',tabwidth,'color',handles.colors.tabfg,'parent',handles.bgtabaxes,'hittest','off');%subright

if handles.ishg2
    set(tablines,'alignvertexcenters','on')
end

controltitleaxes=axes('parent',handles.fig,'units','pixels','position',[120 731 220 40],'visible','off',...
    'xlimmode','manual','ylimmode','manual','visible','off');

% handles.controltitle=text(.15,1,'Control Panel','backgroundcolor','none','color',handles.colors.title,'fontsize',handles.fonts.titlesize,...
%     'horizontalalignment','left','verticalalignment','baseline','parent',handles.tabaxes,...
%     'fontname',handles.fonts.title,'margin',1,'handlevisibility','off','tag','controltitle','hittest','off');
handles.controltitle=text(0,0,'Control Panel','backgroundcolor','none','color',handles.colors.title,'fontsize',1.125*handles.fonts.titlesize,...
    'horizontalalignment','left','verticalalignment','baseline','parent',controltitleaxes,...
    'fontname',handles.fonts.title,'margin',1,'handlevisibility','off','tag','controltitle','hittest','off');


if handles.height<900
    set(handles.controltitle,'visible','off')
end

%draw tabs:
handles.fronttabprofile=[1.1 .7 0.9 1.1 1];
handles.backtabprofile=[0.85 1.05 1.05 0.9 0.65 0.7];

%front row:
[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,1,1);

handles.fronttabbg=patch(x,y,handles.colors.tabbg,'parent',handles.fronttabaxes,'edgecolor','none','tag','tabbg','hittest','off');
handles.fronttablines=line(x,y,'parent',handles.fronttabaxes,'linewidth',tabwidth,'color',handles.colors.tabfg,'hittest','off');

%put text in tabs:
%front row
text(.15,v(1)+0.03,'Manage','fontname',handles.fonts.tab,'color',handles.colors.text,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@managetab,'interruptible','off','tag','managetab')
text(.17,v(2)+0.02,'View','fontname',handles.fonts.tab,'color',handles.colors.text,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@viewtab,'hittest','off','interruptible','off','tag','viewtab')
text(.17,v(3)+0.01,'Format','fontname',handles.fonts.tab,'color',handles.colors.text,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@formattab,'hittest','off','interruptible','off','tag','formattab')
text(.17,v(4)+0.01,'Specialize','fontname',handles.fonts.tab,'color',handles.colors.text,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@exploretab,'hittest','off','interruptible','off','tag','exploretab')
text(.17,v(5)+0.02,'Export','fontname',handles.fonts.tab,'color',handles.colors.text,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.fronttabaxes,'verticalalignment','cap',...
    'buttondownfcn',@exporttab,'interruptible','off','tag','exporttab')
%back row:

[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0);

handles.backtabbg=patch(x,y,handles.colors.tabbg,'parent',handles.backtabaxes,'edgecolor','none','tag','tabbg','hittest','off');
handles.backtablines=line(x,y,'parent',handles.backtabaxes,'linewidth',tabwidth,'color',handles.colors.tabfg,'hittest','off');

%put text in tabs:

text(.17,v(1)+0.01,'Model','fontname',handles.fonts.tab,'color',handles.colors.texttint,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@modeltab,'interruptible','off','tag','modeltab')
text(.17,v(2),'Constrain','fontname',handles.fonts.tab,'color',handles.colors.texttint,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@constraintab,'interruptible','off','tag','constraintab')
text(.17,v(3)+0.01,'Optimize','fontname',handles.fonts.tab,'color',handles.colors.texttint,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@optimizetab,'interruptible','off','tag','optimizetab')
text(.17,v(4)+0.01,'Sample','fontname',handles.fonts.tab,'color',handles.colors.texttint,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@sampletab,'interruptible','off','tag','sampletab')
text(.17,v(5)+0.01,'Row ','fontname',handles.fonts.tab,'color',handles.colors.texttint,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@rowtab,'interruptible','off','tag','rowtab')
text(.17,v(6)+0.01,'Draw','fontname',handles.fonts.tab,'color',handles.colors.texttint,'fontsize',handles.fonts.tabsize,...
    'rotation',90,'parent',handles.backtabaxes,'verticalalignment','cap',...
    'buttondownfcn',@drawtab,'interruptible','off','tag','drawtab')


handles.alltabs=findobj([handles.fronttabaxes,handles.backtabaxes],'type','text');

%make tabs unclickable until the user loads a data set (managedatagui turns
%them on)
set(handles.alltabs,'hittest','off');

handles.fronttabsfront=1; %1 if fronttabs in front, 0 if backtabs in front


%% set up non-tabbed ui controls:
handles.stopbutton=uicontrol('style','pushbutton','position',[handles.width-105 handles.height-26 100 32],...
    'string','debug','fontname','arial narrow','fontsize',20,'foregroundcolor',handles.colors.title,...
    'callback',@stopbutton,'backgroundcolor',handles.colors.buttonbg,'parent',handles.fig);

%an invisible object that can be used to remove the focus from visible
%uicontrols:
handles.dummyobj=uicontrol('style','pushbutton','position',[-100 -100 50 50],'parent',handles.fig);

if handles.height>=900
    gap=9;
else
    gap=3;
end

handles.tablemodebutton=uicontrol('style','pushbutton','position',[18 handles.height-35-pushdown 73 32],...
    'string','Table','callback',@tablemodebutton_callback,'cdata',[],'tooltipstring','View Data Table','parent',handles.fig,...
    'fontsize',12,'tag','tablemodebutton');
handles.newbutton=uicontrol('style','pushbutton','position',[18 handles.height-66-gap-pushdown 32 32],...
    'string','','callback',@newbutton,'cdata',imread('newgraph.bmp'),'tooltipstring','Create New Plot','parent',handles.fig);
handles.zoombutton=uicontrol('style','togglebutton','position',[59 handles.height-66-gap-pushdown 32 32],...
    'string','','callback','','cdata',imread('zoom.bmp'),'tooltipstring',' Zoom ','parent',handles.fig);
handles.showgridbutton=uicontrol('style','togglebutton','position',[18 handles.height-128-3*gap-pushdown 32 32],...
    'string','','callback',@showgridbutton,'value',1,...
    'tooltipstring','Show/Hide Gridlines','cdata',imread('showgrid.bmp'),'parent',handles.fig,...
    'backgroundcolor',handles.colors.togglebuttonbg,'visible','off');
handles.openglbutton=uicontrol('style','pushbutton','position',[59 handles.height-97-2*gap-pushdown 32 32],...
    'string','','callback',@changerenderer_callback,'tooltipstring','Change Renderer','parent',handles.fig,...
    'cdata',imread('raveclock.bmp'),'backgroundcolor',handles.colors.togglebuttonbg);
handles.hideshowbutton=uicontrol('style','pushbutton','position',[18 handles.height-97-2*gap-pushdown 32 32],...
    'string','','callback',@ravehidetabs,'cdata',imread('hideshow.bmp'),...
    'tooltipstring','Hide/Show Control Panel','parent',handles.fig);

if handles.height<=776
    set(handles.tablemodebutton,'position',[20 660 73 32])
    set(handles.newbutton,'position',[85+20 660 32 32]);
    set(handles.zoombutton,'position',[85+40+20 660 32 32]);
    set(handles.showgridbutton,'position',[85+2*40+20 660 32 32]);
    set(handles.openglbutton,'position',[85+3*40+20 660 32 32]);
    set(handles.hideshowbutton,'position',[85+4*40+20 660 32 32]);
end

%% initialize handles.tabobjects so tab changefcn has something to delete
dummyobj=uicontrol('position',[-200 -200 10 10],'style','pushbutton','parent',handles.fig);
handles.tabobjects=dummyobj;
handles.tabobjects=[];

%% load sound effects:
try %64 bit:
    [Y,Fs]=audioread('ravefinish.wav');
    handles.sounds.finish=audioplayer(Y,Fs,24);
catch %32 bit:
    handles.sounds.finish=wavread('ravefinish.wav');
end


%% set up close request function:
set(handles.fig,'closerequestfcn',@ravequitfunction)

%default window keypress fcn:
set(handles.fig,'windowkeypressfcn',@ravedefaultkeypressfcn)
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)

%mousemotion timer:
handles.mousetimer=timer('TimerFcn',[],'startdelay',handles.prefs.mouseoverdelay,'executionmode','singleshot');


%% make help button:
bgcolor=handles.colors.frame;

R=bgcolor(1);
G=bgcolor(2);
B=bgcolor(3);
bgcolor=java.awt.Color(R,G,B);

h = com.mathworks.mwswing.MJButton('Need Help?');
h.setBorder([]);
h.setBackground(bgcolor);
h.setForeground(java.awt.Color.blue);
h.setCursor(java.awt.Cursor(java.awt.Cursor.HAND_CURSOR));
h.setFlyOverAppearance(true);

h.setToolTipText('Open Rave help page');

[jhandle,mhandle] = javacomponent(h,[handles.width-190 handles.height-20 80 18],handles.fig);
set(h, 'ActionPerformedCallback',{@ravedisplayhelpmain});

handles.helpbutton=mhandle;


%%
raveupdateview(handles.fig,[],handles);



%% show splash screen
showsplash=true;
if ismember('-nosplash',varargin)
    showsplash=false;
end


if showsplash
    drawnow;
    pause(0.2); %need to pause because on vista figure window "fades in" and screws up getscreen.
    
    [splash,map,splashalpha]=imread('ravesplashlite.png','png');
    
    wspos=get(handles.wspanel,'position');
    
    splashposition=floor([(wspos(3)-size(splash,2))/2 (wspos(4)-size(splash,1))/2 size(splash,2) size(splash,1)]);
    splashax=axes('units','pixels','position',splashposition,'parent',handles.wspanel,'color','none','visible','off');
    
    %get background pixels, merge with image:
    splashback=getscreen(splashax);
    splashback=splashback.cdata;
    splashback=double(splashback);
    splash=double(splash);
    splashalpha=double(splashalpha);
    splashback=bsxfun(@times,splashback,(255-splashalpha))./255;
    splash=bsxfun(@times,splash,(splashalpha))./255;
    spah=splash+splashback;
    
    image(uint8(spah),'parent',splashax);
    
    if ~handles.ishg2
        set(splashax,'visible','off','position',splashposition-[1 1 0 0]) %correct fact that splashback is 1 pixel off... not really sure why.
    else
        set(splashax,'visible','off','position',splashposition+[1 1 0 0]) %correct fact that splashback is 1 pixel off... not really sure why.
    end
    
    text(635,190,['Version ',ravever],...
        'parent',splashax,'fontname','times','fontsize',12,'hor','right','fontangle','italic','fontweight','bold')
    %Notice to users -- do not modify/remove this line:
    text(635,330,{'Created by Matthew Daskilewicz';'Copyright 2015';'Georgia Tech Research Corporation';'All Rights Reserved'},...
        'parent',splashax,'fontname','times','fontsize',12,'hor','right','fontangle','italic')
    %Notice to users -- do not modify/remove this line:
     text(635,390,{'Distributed under GNU General Public License Ver. 2'},...
         'parent',splashax,'fontname','times','fontsize',11,'hor','right','fontangle','normal')
else
    splashax=[];
end
drawnow;

%Load fonts:
%make html font list (from Yair Altman's book, pg 109)
handles.fonts.listfonts=listfonts;
fontStr = @(font) ['<html><font face= "' font '">' font '</font></html>'];
handles.fonts.htmllistfonts=cellfun(fontStr,handles.fonts.listfonts,'uniform',false);



pause(1.5); %wait to make sure java handles are valid.


%% wrap it up...
%Choose default command line output for rave
handles.output = handles.fig;
% Update handles structure
guidata(handles.fig, handles);

% %Maximize window?
% jf=findjobj(handles.fig,'class','FigureFrame');
% if ~isempty(jf)
%     jf.setExtendedState(handles.prefs.ismaximized);
% end

%% create manage tab
managetab(handles.fig,[]);
handles=guidata(handles.fig);

%% finish up...
set(handles.fig,'windowscrollwheelfcn',@ravewheelscroll,'resizefcn',@raveresizefcn) %note, dont set resizefcn sooner or it will try to run before all objects have been created.

%get initial axiscanvas for java stuff:
handles.axiscanvas=ravegetaxiscanvas(handles.fig,'update');

guidata(handles.fig,handles);
%make font metrics objects:
handles=ravemakefontmetrics(handles.fig,[]);


if handles.prefs.isresizable
    set(handles.fig,'resize','on') %note: this triggers resize function.
end

% Update handles structure
guidata(handles.fig, handles);

%load context menus:
ravemakecontextmenus(handles.fig,[]);
handles=guidata(handles.fig);

%% do java stuff:
drawnow; %Do not remove... this needs to be here or java stuff on manage tab gets screwed up. (Not sure why)


handles=guidata(handles.fig);
handles.jvscroll=findjobj(handles.vscroll);
handles.jhscroll=findjobj(handles.hscroll);
try %#ok<TRYNC>
    handles.jvscroll.MouseDraggedCallback = {@vertscroll,handles.fig};
    handles.jhscroll.MouseDraggedCallback = {@horizscroll,handles.fig};
end

guidata(handles.fig,handles);
delete(splashax)
if ismember('Magneto',handles.fonts.listfonts);
    set(handles.infobar,'string',['Welcome to the \fontsize{18}\fontname{magneto}\color[rgb]{',num2str(handles.colors.text),'}rave\fontsize{14}\fontname{' handles.fonts.tab '}\color[rgb]{',num2str(handles.colors.text),'}, click "Manage Data Sets" to begin.']);
else
    set(handles.infobar,'string','Welcome to the Rave, click "Manage Data Sets" to begin.');
end

drawnow;

%% check for updates:
if handles.prefs.checkforupdates
    ravecheckforupdates;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% callbacks...
function stopbutton(hObject,eventdata)  %#ok<INUSD>
handles=guidata(hObject);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
datasets=raveguidata(handles.fig,'datasets'); %#ok<NASGU>
% textdata=raveguidata(handles.fig,'textdata'); %#ok<NASGU>
vectordata=raveguidata(handles.fig,'vectordata'); %#ok<NASGU>
keyboard

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function newbutton(hObject,eventdata)
handles=guidata(hObject);
uicontrol(handles.dummyobj);
ravenewgraph(handles.fig,[])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function changerenderer_callback(src,ev) %#ok<INUSD>
handles=guidata(src);

rend=get(handles.fig,'renderer');

switch lower(rend)
    case 'painters'
        
        if ~handles.ishg2
            set(handles.fig,'renderer','zbuffer');
            set(handles.infobar,'string','Renderer set to ZBuffer')
        else
            set(handles.fig,'renderer','opengl');
            set(handles.infobar,'string','Renderer set to Open GL')
        end
        
    case 'zbuffer'
        %check for incompatible graphs:
        dopainters=false;
        for i=1:numel(handles.allgraphs)
            if ishandle(handles.allgraphs(i)) && strcmp(get(handles.allgraphs(i),'userdata'),'density')
                dopainters=true;
            end
        end
        
        if dopainters
            set(handles.infobar,'string','Open GL is incompatible with Density Plot; switching to Painters')
            set(handles.fig,'renderer','painters');
        else
            set(handles.fig,'renderer','opengl');
            set(handles.infobar,'string','Renderer set to Open GL')
        end
    case 'opengl'
        set(handles.fig,'renderer','painters');
        set(handles.infobar,'string','Renderer set to Painters')
end

drawnow;

%update java graphics stuff:

handles.axiscanvas=ravegetaxiscanvas(handles.fig,'update');


guidata(handles.fig,handles);
clear functions %<--I dont know which function is causing problems, but something inmem is causing nullpointerexceptions
%in superiorlinewrap after the renderer is changed from opengl to painters.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function tablemodebutton_callback(src,ev)
handles=guidata(src);

if handles.tablemode %go from table to workspace
    handles.activesheet=handles.lastsheet;
    handles.lastsheet=0;
    guidata(handles.fig,handles);
    set(handles.pageedit,'string',num2str(handles.activesheet))
    showhideworkspace(handles.fig,[]);
else %go from workspace to table:
    handles.lastsheet=handles.activesheet;
    handles.activesheet=0;
    guidata(handles.fig,handles);
    set(handles.pageedit,'string',num2str(handles.activesheet))
    showhideworkspace(handles.fig,[]);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%




