function [yout,yout_samples]=raveevaluatefunction(src,d,ycol_original,data,handles,ignore)
%evaluates the function for the columns in "ycol" in dataset "d" using the input data stored in the matrix
%"data".
%if ycol is a vector, evaluates all functions in ycol and
%returns all responses as columns of a matrix in the order they appear in ycol. Rave will attempt to avoid
% calling the same function multiple times . All responses must share the same input data (data)
%
%5th argument, handles, is optional. Use it to reduce overhead of calling
%guidata.
%
%6th optional arguement, ignore, is a list of column numbers that should
%not be evaluated as intermediate functions. Instead they will use whatever
%values are in the corresponding columns of "data". (If you include 6th
%input, you must include handles as the 5th input).
%
%2nd output, yout_samples, is a cell row-vector with as many columns as yout. If yout(i) is
% deterministic (not noisy), yout_samples{i} is empty. Otherwise, yout_smaples{i} is itself a cell
% array -- a column vector with as many rows as "data". Each cell in yout_samples{i}{g} is a column vector
% of the monte carlo samples of the g-th design point. Cell arrays are used so that each design could
% potentially involve a different number of samples. (Although the function that calls this function
% may impose additional restrictions)
%
%NOTE: this function does not enforce discrete variable values. That must be handled by whatever
%calls this.
%
%NOTE: if any of the inputs are strings, data will be a cell array instead
%of a double array.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu
%

%Explanation of function attributes:
% * onetomany - This indicates a "noisy" function that has its own built-in sampling method rather than
%               using Rave's random variable system.

ycol=unique(ycol_original);
yout_samples=cell(size(data,1),numel(ycol));

%determine in what order to return results:
[ycol,y_order]=sort(ycol);

%remove NaNs, which can be used in ycol_original to insert blank columns in
%the output:
% y_order=y_order(~isnan(ycol));
ycol=ycol(~isnan(ycol));

%handle empty cases:
if isempty(ycol);
    yout=zeros(size(data,1),0);
    return;
elseif size(data,1)==0
    yout=zeros(0,numel(ycol));
    return
else
    yout=zeros(size(data,1),numel(ycol_original));
end

if nargin<5
    handles=guidata(src);
end

if nargin<6
    ignore=[];
end

prefs=handles.preferences{d};
targets=handles.targetvalues{d};

%get unique rows:
[uniquedata,otherrowidx,rowidx]=unique(data,'rows');

%deterine dependency. ycol_sorted also includes any intermediate variables
%that need to be evaluted to calculate ycol
ycol_sorted=sort(ycol,'descend');
f=numel(ycol_sorted);
while f<=numel(ycol_sorted)
    thesefunvars=handles.funvars{d}{ycol_sorted(f)};
    thesefunvars=setdiff(thesefunvars,ignore); %Prevent ignored variables and their upstream vars from showing up
    ycol_sorted=union(ycol_sorted,thesefunvars(handles.isfunction{d}(thesefunvars)));
    f=f+1;
end


%If any variables are noise, quit this and call the noisy function evaluator:
isnoisy=false(size(ycol_sorted));
for g=1:numel(ycol_sorted)
    index=ravegetindependentvars(handles,d,1,ycol_sorted(g),[]);
    if any(handles.isnoise{d}(index)) && ~ismember('onetomany',handles.funattributes{d}{g});
        isnoisy(g)=true;
    end
end

isonetomany = false(size(ycol_sorted));
for g=1:numel(ycol_sorted)
    if ismember('onetomany',handles.funattributes{d}{ycol_sorted(g)});
        isonetomany(g)=true;
    end
end

isdeterministic = ~(isnoisy | isonetomany);
isnoisy = isnoisy & ~isonetomany;

%The non-noisy functions are evaluated first. This is because it is
%possible for a noisy function to have non-noisy intermediate functions as
%inputs, but not for a non-noisy function to have noisy inputs.
ycol_noisy=ycol_sorted(isnoisy);
ycol_onetomany = ycol_sorted(isonetomany);
ycol_deterministic=ycol_sorted(isdeterministic);

isdeterministic_orig = isdeterministic(ismember(ycol_sorted,ycol_original));
isnoisy_orig=isnoisy(ismember(ycol_sorted,ycol_original));
isonetomany_orig=isonetomany(ismember(ycol_sorted,ycol_original));


is_orig = ismember(ycol_sorted,ycol_original);



%% EVALUATE NON-NOISY FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(ycol_deterministic)
    name=handles.funnames{d}(ycol_deterministic);
    narg=handles.nargout{d}(ycol_deterministic);

        %enlarge (if necessary) data to have as many columns as largest index:
        if size(uniquedata,2)<max(ycol_deterministic)
            uniquedata=[uniquedata zeros(size(uniquedata,1),max(ycol_deterministic)-size(uniquedata,2))];
        end
        
        isfuns=find(handles.isfunction{d}(ycol_deterministic));
        if ~isempty(isfuns)
            
            [junk,index]=unique(name(isfuns)); %get index of first occurence of unique funs
            funname=name(isfuns);
            uniquefuns = funname(sort(index)); %extract unique funs without changing their evaluation order.
            
            for g=1:numel(uniquefuns);
                thisfun=uniquefuns{g};
                isthisfun=find(strcmp(name,thisfun)); %responses matching current function
                xfun=str2func(thisfun);
                alloutputs=cell(1,nargout(xfun));
                
                %What is the highest-index input variable to this function?
                %Use this to supply only as many columns as needed, in
                %case the function would react badly to having too many
                %columns included in the input:
                funfunfun = strcmp(handles.funnames{d},uniquefuns{g});
                xmax = max([handles.funvars{d}{funfunfun}]);
                
                
                
                switch handles.funclass{d}{ycol_deterministic(isfuns(g))}
                    case 'matlab_generic' %not used anymore but keep here for backward compatibility
                        if nargin(xfun)==3
                            [alloutputs{:}]=xfun(uniquedata(:,1:xmax),prefs,targets);
                        else
                            [alloutputs{:}]=xfun(uniquedata(:,1:xmax));
                        end
                    case 'matlab_singleinput';
                        [alloutputs{:}]=xfun(uniquedata(:,1:xmax));
                    case 'matlab_threeinput'; %
                        [alloutputs{:}]=xfun(uniquedata(:,1:xmax),prefs,targets);
                    case 'matlab_varnames'; %each variable in its own column
                        udat=uniquedata(:,handles.funvars{d}{ycol_deterministic(isfuns(g))});
                        udat=mat2cell(udat,size(udat,1),ones(1,size(udat,2)));
                        [alloutputs{:}]=xfun(udat{:});
                    case 'independent'
                        alloutputs={uniquedata(:,ycol_deterministic(isfuns(g)))};
                    case 'bbnbnt' %bayesian network toolbox bbn
                        alloutputs=raveevaluatebbnbnt(src,d,ycol,data,handles);
                end
                
                alloutputs=cell2mat(alloutputs);
                uniquedata(:,ycol_deterministic(isthisfun))=alloutputs(:,narg(isthisfun));
            end
        end
        y=uniquedata(:,ycol_deterministic);
%     end
    yout(:,isdeterministic_orig)=y(rowidx,is_orig(isdeterministic));
end


%% EVALUATE OneToMany FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%These use "data" instead of "uniquedata" because nondeterminsticness inside of the user's function could result in different
%answers when analyzing the same point multiple times, andthis may be what the user intended.

if ~isempty(ycol_onetomany)
    name=handles.funnames{d}(ycol_onetomany);
    narg=handles.nargout{d}(ycol_onetomany);
    
    %enlarge (if necessary) data to have as many columns as largest index:
    if size(data,2)<max(ycol_onetomany)
        data=[data zeros(size(data,1),max(ycol_onetomany)-size(data,2))];
    end
    
    isfuns=find(handles.isfunction{d}(ycol_onetomany));
    if ~isempty(isfuns)
        
        [junk,index]=unique(name(isfuns)); %get index of first occurence of unique funs
        funname=name(isfuns);
        uniquefuns = funname(sort(index)); %extract unique funs without changing their evaluation order.
        
        for g=1:numel(uniquefuns);
            thisfun=uniquefuns{g};
            isthisfun=find(strcmp(name,thisfun)); %responses matching current function
            xfun=str2func(thisfun);
            alloutputs=cell(1,nargout(xfun));
            
            funfunfun = strcmp(handles.funnames{d},uniquefuns{g});
            xmax = max([handles.funvars{d}{funfunfun}]);

            switch handles.funclass{d}{ycol_onetomany(isfuns(g))}
                case 'matlab_generic' %not used anymore but keep here for backward compatibility
                    if nargin(xfun)==3
                        [alloutputs{:}]=xfun(data(:,1:xmax),prefs,targets);
                    else
                        [alloutputs{:}]=xfun(data(:,1:xmax));
                    end
                case 'matlab_singleinput';
                    [alloutputs{:}]=xfun(data(:,1:xmax));
                case 'matlab_threeinput'; %
                    [alloutputs{:}]=xfun(data(:,1:xmax),prefs,targets);
                case 'matlab_varnames'; %each variable in its own column
                    udat=data(:,handles.funvars{d}{ycol_onetomany(isfuns(g))});
                    udat=mat2cell(udat,size(udat,1),ones(1,size(udat,2)));
                    [alloutputs{:}]=xfun(udat{:});
                case 'independent'
                    alloutputs={data(:,ycol_onetomany(isfuns(g)))};
                case 'bbnbnt' %bayesian network toolbox bbn
                    alloutputs=raveevaluatebbnbnt(src,d,ycol,data,handles);
            end
            
            %Store samples:
            yout_samples(:,isonetomany_orig) = alloutputs(:,narg(isthisfun));
            
            %Calculate statistics:
            stats=handles.aggregatingstatistic{d}(ycol_onetomany);
            y_stat=accumulatestatistics(yout_samples(:,isonetomany_orig),stats);

            data(:,ycol_onetomany(isthisfun))=y_stat(:,isthisfun);
        end
    end
    y=data(:,ycol_onetomany);
    yout(:,isonetomany_orig)=y;
end


%% EVALUATE NOISY FUNCTIONS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if ~isempty(ycol_noisy) %need to check nargout and call with appropriate # of outputs here so that we can check nargout
    %                    inside raveevaluatenoisyfunction
    if nargout==2
        [yout(:,isnoisy_orig),yout_samples(:,isnoisy_orig)]=raveevaluatenoisyfunction(src,d,ycol_noisy(ismember(ycol_noisy,ycol_original)),data,yout,handles,ignore);
    else
        [yout(:,isnoisy_orig)]=raveevaluatenoisyfunction(src,d,ycol_noisy(ismember(ycol_noisy,ycol_original)),data,yout,handles,ignore);
    end
end


%% Put results in desired output order:
% [junk,y_order]=ismember(ycol_original,ycol_sorted);
% y_order=y_order(~isnan(ycol_original));

[junk,y_order]=sort(ycol_original);
yout=yout(:,y_order);
yout_samples=yout_samples(:,y_order);

%yout(:,~isnan(ycol_original))=yout(:,y_order);
% yout_all(~isnan(ycol_original))=yout_all(y_order);


function y_stat=accumulatestatistics(y,stats)
%Combine monte carlo runs into a single statistic. Note that this function
%differs from the equivalent function in raveevaluatenoisyfunction because
%this one must act on cell arrays. 

for g=size(y,2):-1:1
    switch stats{g}{1}
        case 'mean'
            y_stat(:,g)=cellfun(@mean,y{g});
        case 'median'
            y_stat(:,g)=cellfun(@median,y{g});
        case 'stddev'
            y_stat(:,g)=cellfun(@std,y{g});
        case 'variance'
            y_stat(:,g)=cellfun(@var,y{g});
            
    end
end


